local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local TooltipsModule = idTweaks:NewModule('Tooltips', 'AceConsole-3.0', 'AceHook-3.0')

TooltipsModule.DEBUG = false

function TooltipsModule:Debug(...)
  if TooltipsModule.DEBUG then
    self:Print(...)
  end
end

function TooltipsModule:setDefaults()
  self:Debug('TooltipsModule:setDefaults')

  self.defaults = {
    profile = {
      enabled = true,

      anchor = {
        value = 'ANCHOR_CURSOR'
      }
    }
  }
end

function TooltipsModule:setDatabase()
  self:Debug('TooltipsModule:setDatabase')

  self.db = idTweaks.db:RegisterNamespace('Tooltips', self.defaults)
end

function TooltipsModule:setOptions()
  self:Debug('TooltipsModule:setOptions')

  self.options = {
    name = 'Tooltips',
    handler = self,
    type = 'group',
    args = {
      enabled = {
        order = 1,
        name = 'Enabled',
        desc = 'Enables / Disables the Module',
        handler = self,
        type = 'toggle',
        get = function()
          return self.db.profile.enabled
        end,
        set = function(_, value)
          self.db.profile.enabled = value
          self:check()
        end,
      },

      anchor = {
        name = 'Anchor',
        type = 'select',
        values = {
          ['ANCHOR_NONE'] = 'Default',
          ['ANCHOR_CURSOR'] = 'Cursor',
          ['ANCHOR_TOPLEFT'] = 'Top Left',
          ['ANCHOR_TOPRIGHT'] = 'Top Right',
          ['ANCHOR_BOTTOMLEFT'] = 'Bottom Left',
          ['ANCHOR_BOTTOMRIGHT'] = 'Bottom Right',
        },
        disabled = function()
          return not self.db.profile.enabled
        end,
        get = function()
          return self.db.profile.anchor.value
        end,
        set = function(_, value)
          self.db.profile.anchor.value = value
        end,
      }
    }
  }
end

function TooltipsModule:registerOptions()
  self:Debug('TooltipsModule:registerOptions')

  LibStub('AceConfig-3.0'):RegisterOptionsTable('idTweaks-Tooltips', self.options)
  LibStub('AceConfigDialog-3.0'):AddToBlizOptions('idTweaks-Tooltips', 'Tooltips', 'idTweaks')
end

function TooltipsModule:OnInitialize()
  self:Debug('TooltipsModule:OnInitialize')

  self:setDefaults()
  self:setDatabase()
  self:setOptions()
end

function TooltipsModule:OnEnable()
  self:Debug('TooltipsModule:OnEnable')

  self:registerOptions()

  self:check()
end

function TooltipsModule:OnDisable()
  self:Debug('TooltipsModule:OnDisable')

  self:disable()
end

function TooltipsModule:check()
  self:Debug('TooltipsModule:check')

  if self.db.profile.enabled then
    self:enable()
  else
    self:disable()
  end
end

function TooltipsModule:enable()
  self:Debug('TooltipsModule:enable')

  self:SecureHook('GameTooltip_SetDefaultAnchor')
end

function TooltipsModule:disable()
  self:Debug('TooltipsModule:disable')

  self:Unhook('GameTooltip_SetDefaultAnchor')
end

function TooltipsModule:GameTooltip_SetDefaultAnchor(tooltip, selfObject)
  self:Debug('TooltipsModule:GameTooltip_SetDefaultAnchor')

  tooltip:SetOwner(selfObject, TooltipsModule.db.profile.anchor.value)
end
