local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local UnitFramesModule = idTweaks:NewModule('UnitFrames', 'AceConsole-3.0', 'AceHook-3.0')
local LibSharedMedia = LibStub('LibSharedMedia-3.0')

local _, ns = ...
local oUF = ns.oUF

local TL, TC, TR = 'TOPLEFT', 'TOP', 'TOPRIGHT'
local ML, MC, MR = 'LEFT', 'CENTER', 'RIGHT'
local BL, BC, BR = 'BOTTOMLEFT', 'BOTTOM', 'BOTTOMRIGHT'

local STATUS_BAR_TEXTURE = 'Interface/TargetingFrame/UI-StatusBar'

UnitFramesModule.DEBUG = false

function UnitFramesModule:Debug(...)
  if UnitFramesModule.DEBUG then
    self:Print(...)
  end
end

local sharedMediaStatusBars = LibSharedMedia:HashTable('statusbar')
local sharedMediaStatusBarsReversed = idTweaks.table.reverse(LibSharedMedia:HashTable('statusbar'))
local sharedMediaBackgrounds = LibSharedMedia:HashTable('background')
local sharedMediaBackgroundsReversed = idTweaks.table.reverse(LibSharedMedia:HashTable('background'))

function UnitFramesModule:getDBForUnit(unit)
  local dbUnit = UnitFramesModule.db.profile[unit]
  local dbDefault = UnitFramesModule.db.profile.default

  return dbUnit.defaults and dbDefault or dbUnit
end

function UnitFramesModule:setDefaults()
  self:Debug('UnitFramesModule:setDefaults')

  -- self:Debug(LibSharedMedia:HashTable('background'))

  local default = {
    frame = {
      -- TODO: Move top level size to here
      size = {
      },

      -- TODO: Implement.
      positioning = {
        from = 'CENTER',
        target = 'UIParent',
        to = 'CENTER',
        x = 0,
        y = 0,
      },

      -- TODO: Implement.
      backgroundTexture = '',

      -- TODO: Implement.
      borderTexture = '',
      borderInsets = {
        top = 0,
        left = 0,
        right = 0,
        bottom = 0,
      },
    },

    size = {
      width = 600,
      height = 40,
      padding = 5,
      margin = 5,
    },

    healthBar = {
      texture = 'Interface\\TargetingFrame\\UI-StatusBar',

      -- TODO: Rename to backgroundTexture
      background = 'Interface\\DialogFrame\\UI-DialogBox-Background-Dark',

      -- TODO: Implement.
      borderTexture = '',

      -- TODO: Implement.
      borderInsets = {
        top = 0,
        left = 0,
        right = 0,
        bottom = 0,
      },
    },
    powerBar = {
      texture = 'Interface\\TargetingFrame\\UI-StatusBar',

      -- TODO: Rename to backgroundTexture
      background = 'Interface\\DialogFrame\\UI-DialogBox-Background-Dark',

      -- TODO: Implement.
      borderTexture = '',

      -- TODO: Implement.
      borderInsets = {
        top = 0,
        left = 0,
        right = 0,
        bottom = 0,
      },
    },
    additionalPowerBar = {
      texture = 'Interface\\TargetingFrame\\UI-StatusBar',

      -- TODO: Rename to backgroundTexture
      background = 'Interface\\DialogFrame\\UI-DialogBox-Background-Dark',

      -- TODO: Implement.
      borderTexture = '',

      -- TODO: Implement.
      borderInsets = {
        top = 0,
        left = 0,
        right = 0,
        bottom = 0,
      },
    },
    text = {
      -- TODO: Implement.
      name = {
        padding = 5,
        font = '',
        fontColor = { r=255, g=255, b=255 },
        tag = '',
      },

      -- TODO: Implement.
      health = {
        padding = 5,
        font = '',
        fontColor = { r=255, g=255, b=255 },
        tag = '',
      },

      -- TODO: Implement.
      power = {
        padding = 5,
        font = '',
        fontColor = { r=255, g=255, b=255 },
        tag = '',
      },

      -- TODO: Implement.
      classPower = {
        padding = 5,
        font = '',
        fontColor = { r=255, g=255, b=255 },
        tag = '',
      },
    },
  }

  self.defaults = {
    profile = {
      enabled = true,

      default = default,

      pet = idTweaks.table.merge(default, {
        defaults = true
      }),

      player = idTweaks.table.merge(default, {
        defaults = true
      }),

      target = idTweaks.table.merge(default, {
        defaults = true
      }),

      targettarget = idTweaks.table.merge(default, {
        defaults = true
      }),

      party = idTweaks.table.merge(default, {
        defaults = true
      }),
    },
  }
end

function UnitFramesModule:setDatabase()
  self:Debug('UnitFramesModule:setDatabase')

  self.db = idTweaks.db:RegisterNamespace('UnitFrames', self.defaults)
end

function UnitFramesModule:getUnitOptions(unitName, unit)
  local output = {
    name = unitName,
    handler = self,
    type = 'group',
    args = {
      enabled = {
        order = 1,
        name = 'Enabled',
        desc = 'Enables / Disables the Module',
        handler = self,
        type = 'toggle',
        get = function()
          return self.db.profile.enabled
        end,
        set = function(_, value)
          self.db.profile.enabled = value
          self:check()
        end,
      },

      size = {
        name = 'Size',
        desc = 'Control the size of the frame.',
        type = 'group',
        handler = self,
        disabled = function()
          return unit ~= 'default' and self.db.profile[unit].defaults
        end,
        args = {
          width = {
            name = 'Width',
            type = 'range',
            min = 100,
            max = 1000,
            step = 1,
            bigStep = 10,
            get = function()
              return self.db.profile[unit].size.width
            end,
            set = function(_, value)
              self.db.profile[unit].size.width = value
              self.size:positionElements(unit)
            end
          },
          height = {
            name = 'Height',
            type = 'range',
            min = 5,
            max = 100,
            step = 1,
            bigStep = 1,
            get = function()
              return self.db.profile[unit].size.height
            end,
            set = function(_, value)
              self.db.profile[unit].size.height = value
              self.size:positionElements(unit)
            end
          },
          padding = {
            name = 'Padding',
            desc = 'Spacing between elements in the frame.',
            type = 'range',
            min = 0,
            max = 50,
            step = 1,
            bigStep = 1,
            get = function()
              return self.db.profile[unit].size.padding
            end,
            set = function(_, value)
              self.db.profile[unit].size.padding = value
              self.size:positionElements(unit)
            end
          },
          margin = {
            name = 'Margin',
            desc = 'Spacing between this frame and the next.',
            type = 'range',
            min = 0,
            max = 50,
            step = 1,
            bigStep = 1,
            get = function()
              return self.db.profile[unit].size.margin
            end,
            set = function(_, value)
              self.db.profile[unit].size.margin = value
              self.size:positionFrame(unit)
            end
          },
        },
      },
      positioning = {
        name = 'Positioning',
        desc = 'Control the positioning of the frame.',
        handler = self,
        disabled = function() return unit ~= 'default' and self.db.profile[unit].defaults end,
        type = 'group',
        args = {
        },
      },
      healthBar = {
        name = 'Health Bar',
        handler = self,
        disabled = function()
          return unit ~= 'default' and self.db.profile[unit].defaults
        end,
        type = 'group',
        args = {
          texture = {
            name = 'Texture',
            type = 'select',
            dialogControl = 'LSM30_Statusbar',
            values = LibSharedMedia:HashTable('statusbar'),
            get = function()
              return sharedMediaStatusBarsReversed[self.db.profile[unit].healthBar.texture]
            end,
            set = function(_, value)
              self.db.profile[unit].healthBar.texture = sharedMediaStatusBars[value]
              self.healthBar:setTexture(unit)
            end
          },
          background = {
            name = 'Texture',
            type = 'select',
            dialogControl = 'LSM30_Background',
            values = LibSharedMedia:HashTable('background'),
            get = function()
              return sharedMediaBackgroundsReversed[self.db.profile[unit].healthBar.background]
            end,
            set = function(_, value)
              self.db.profile[unit].healthBar.background = sharedMediaBackgrounds[value]
              self.healthBar:setBackground(unit)
            end
          },
        },
      },
      powerBar = {
        name = 'Power Bar',
        handler = self,
        disabled = function()
          return unit ~= 'default' and self.db.profile[unit].defaults
        end,
        type = 'group',
        args = {
          width = {
            name = 'Texture',
            type = 'select',
            dialogControl = 'LSM30_Statusbar',
            values = LibSharedMedia:HashTable('statusbar'),
            get = function()
              return sharedMediaStatusBarsReversed[self.db.profile[unit].powerBar.texture]
            end,
            set = function(_, value)
              self.db.profile[unit].powerBar.texture = sharedMediaStatusBars[value]
              self.powerBar:setTexture(unit)
            end
          },
          background = {
            name = 'Texture',
            type = 'select',
            dialogControl = 'LSM30_Background',
            values = LibSharedMedia:HashTable('background'),
            get = function()
              return sharedMediaBackgroundsReversed[self.db.profile[unit].powerBar.background]
            end,
            set = function(_, value)
              self.db.profile[unit].powerBar.background = sharedMediaBackgrounds[value]
              self.powerBar:setBackground(unit)
            end
          },
        },
      },
      additionalPowerBar = {
        name = 'Additional Power Bar',
        handler = self,
        disabled = function()
          return unit ~= 'default' and self.db.profile[unit].defaults
        end,
        type = 'group',
        args = {
          width = {
            name = 'Texture',
            type = 'select',
            dialogControl = 'LSM30_Statusbar',
            values = LibSharedMedia:HashTable('statusbar'),
            get = function()
              return sharedMediaStatusBarsReversed[self.db.profile[unit].additionalPowerBar.texture]
            end,
            set = function(_, value)
              self.db.profile[unit].additionalPowerBar.texture = sharedMediaStatusBars[value]
              self.additionalPowerBar:setTexture(unit)
            end
          },
          background = {
            name = 'Texture',
            type = 'select',
            dialogControl = 'LSM30_Background',
            values = LibSharedMedia:HashTable('background'),
            get = function()
              return sharedMediaBackgroundsReversed[self.db.profile[unit].additionalPowerBar.background]
            end,
            set = function(_, value)
              self.db.profile[unit].additionalPowerBar.background = sharedMediaBackgrounds[value]
              self.additionalPowerBar:setBackground(unit)
            end
          },
        },
      },
      text = {
        name = 'Text',
        handler = self,
        disabled = function()
          return unit ~= 'default' and self.db.profile[unit].defaults
        end,
        type = 'group',
        args = {
        },
      },
    },
  }

  if unit == 'default' then
  else
    output.args.defaults = {
      name = 'Defaults',
      type = 'toggle',
      handler = self,
      get = function()
        return self.db.profile[unit].defaults
      end,
      set = function(_, value)
        self.db.profile[unit].defaults = value
      end
    }
  end

  return output
end

function UnitFramesModule:setOptions()
  self:Debug('UnitFramesModule:setOptions')

  self.options = {
    name = 'UnitFrames',
    handler = self,
    type = 'group',
    args = {
      enabled = {
        name = 'Enabled',
        type = 'toggle',
        handler = self,
        get = function()
  self:Debug('UnitFramesModule:setOptions:get', self.db.profile.default.enabled)
          return self.db.profile.default.enabled
        end,
        set = function(_, value)
          self.db.profile.default.enabled = value
  self:Debug('UnitFramesModule:setOptions:set', self.db.profile.default.enabled)
        end
      },
      default = self:getUnitOptions('Default', 'default'),
      pet = self:getUnitOptions('Pet', 'pet'),
      player = self:getUnitOptions('Player', 'player'),
      target = self:getUnitOptions('Target', 'target'),
      targettarget = self:getUnitOptions('Target of Target', 'targettarget'),
      party = self:getUnitOptions('Party', 'party'),
    }
  }
end

function UnitFramesModule:registerOptions()
  self:Debug('UnitFramesModule:registerOptions')

  LibStub('AceConfig-3.0'):RegisterOptionsTable('idTweaks-UnitFrames', self.options)
  LibStub('AceConfigDialog-3.0'):AddToBlizOptions('idTweaks-UnitFrames', 'UnitFrames', 'idTweaks')
end

function UnitFramesModule:OnInitialize()
  self:Debug('UnitFramesModule:OnInitialize')

  self:setDefaults()
  self:setDatabase()
  self:setOptions()
end

function UnitFramesModule:OnEnable()
  self:Debug('UnitFramesModule:OnEnable')

  self:registerOptions()

  if self.db.profile.default.enabled then
  self:Debug('UnitFramesModule:OnEnable:Enabled')

    oUF:RegisterStyle('idTweaks', self.layouts.default)
    oUF:Factory(self.layouts.factory)
  end
end

function UnitFramesModule:OnDisable()
  self:Debug('UnitFramesModule:OnDisable')
  self:Debug('UnitFramesModule:OnDisable: TODO: Disable oUF')
end

-- Utility functions.
function UnitFramesModule:getDefaultOrUnitSetter(fn)
  return function(self, unit, ...)
    if unit == 'default' then
      for name, frame in pairs(UnitFramesModule.frames) do
        if UnitFramesModule.db.profile[name].defaults then
          fn(name, ...)
        end
      end
    else
      fn(unit, ...)
    end
  end
end

function UnitFramesModule:shouldFrameHaveAdditionalPower(unit)
  local _, _, classIndex = UnitClass(unit)

  if classIndex == 1 then
  elseif classIndex == 2 then
  end

  --   Death Knight:  PowerBar and Runes (text)
  --   Demon Hunter:  PowerBar
  --   Druid:         PowerBar and AdditionalPowerBar (Moonkin)
  -- * Mage           PowerBar and Arcane Charges (text)
  -- * Monk           PowerBar and Chi (text)
  -- * Paladin        PowerBar Holy Power (text)
  --   Priest         Mana ???
  -- * Rogue          PowerBar Combo Points (text)
  --   Shaman         PowerBar and AdditionalPowerBar ???
  -- * Warlock        PowerBar and Soul Shards (text)
  --   Warrior        PowerBar
end

-- Keeps the instances of the frames.
UnitFramesModule.frames = {}

-- Size functions.
UnitFramesModule.size = {}

UnitFramesModule.size.positionElements = UnitFramesModule:getDefaultOrUnitSetter(function(unit, frame)
  frame = frame or UnitFramesModule.frames[unit]
  local db = UnitFramesModule:getDBForUnit(unit)

  --[[
    total     = 120
    padding   =   5
    healthBar =  60
    padding   =   5
    powerBar  =  20
    padding   =   5
    aPowerBar =  20
    padding   =   5

    120 (total) - 25 (padding) = 95

    95 / 2 = 47.5

    47.5 / 3

    total     = 145
  ]]

  frame:SetWidth(db.size.width)
  frame:SetHeight(db.size.height)

  local widthMinusPadding = db.size.width - db.size.padding * 2
  local heightMinusPadding = db.size.height - db.size.padding * 3 -- 2 statusbars

  frame.Health:SetPoint(TL, frame, TL, db.size.padding, -db.size.padding)
  frame.Health:SetWidth(widthMinusPadding)
  frame.Health:SetHeight(heightMinusPadding / 2)

  frame.Power:SetPoint(TC, frame.Health, BC, 0, -db.size.padding)
  frame.Power:SetWidth(widthMinusPadding)
  frame.Power:SetHeight(heightMinusPadding / 2)

  -- frame.AdditionalPower:SetPoint(TC, frame.Power, BC, 0, -db.size.padding)
  -- frame.AdditionalPower:SetWidth(widthMinusPadding)
  -- frame.AdditionalPower:SetHeight(heightMinusPadding / 2 / 2)

  frame.Buffs.size = db.size.width / 8
  frame.Buffs.initialAnchor = BL
  frame.Buffs['growth-x'] = 'RIGHT'
  frame.Buffs['growth-y'] = 'UP'
  frame.Buffs:SetSize(8 * frame.Buffs.size, 2 * frame.Buffs.size)
  frame.Buffs:SetPoint(BL, frame, TL, 0, db.size.padding)

  frame.Debuffs.size = db.size.width / 8
  frame.Debuffs.initialAnchor = TL
  frame.Debuffs['growth-x'] = 'RIGHT'
  frame.Debuffs['growth-y'] = 'DOWN'
  frame.Debuffs:SetSize(8 * frame.Debuffs.size, 2 * frame.Debuffs.size)
  frame.Debuffs:SetPoint(TL, frame, BL, 0, -db.size.padding)
end)

UnitFramesModule.size.positionFrame = UnitFramesModule:getDefaultOrUnitSetter(function(unit, frame)
  frame = frame or UnitFramesModule.frames[unit]
  local db = UnitFramesModule:getDBForUnit(unit)

  if unit == 'player' then
    frame:SetPoint(MR, UIParent, MC, -db.size.margin / 2, -200)
  elseif unit == 'target' then
    frame:SetPoint(ML, UIParent, MC, db.size.margin / 2, -200)
  elseif unit == 'targettarget' then
    frame:SetPoint(ML, UnitFramesModule.frames.target, MR, db.size.margin, 0)
  elseif unit == 'pet' then
    print('pet!', MR, ML, -db.size.margin, 0, UnitFramesModule.frames.player)
    frame:SetPoint(MR, UnitFramesModule.frames.player, ML, -db.size.margin, 0)

  -- elseif unit == 'party' then
  --   frame:SetPoint(TC, UnitFramesModule.frames.pet, BC, 0, -margin)
  end
end)

UnitFramesModule.healthBar = {}

UnitFramesModule.healthBar.setTexture = UnitFramesModule:getDefaultOrUnitSetter(function(unit, frame)
  frame = frame or UnitFramesModule.frames[unit]
  local db = UnitFramesModule:getDBForUnit(unit)

  -- self:Debug(db.healthBar.texture)

  frame.Health:SetStatusBarTexture(db.healthBar.texture)

end)

UnitFramesModule.healthBar.setBackground = UnitFramesModule:getDefaultOrUnitSetter(function(unit, frame)
  frame = frame or UnitFramesModule.frames[unit]
  local db = UnitFramesModule:getDBForUnit(unit)

  -- self:Debug(db.healthBar.background)

  frame.Health.bg:SetTexture(db.additionalPowerBar.background)
end)

UnitFramesModule.powerBar = {}

UnitFramesModule.powerBar.setTexture = UnitFramesModule:getDefaultOrUnitSetter(function(unit, frame)
  frame = frame or UnitFramesModule.frames[unit]
  local db = UnitFramesModule:getDBForUnit(unit)

  frame.Power:SetStatusBarTexture(db.powerBar.texture)
end)

UnitFramesModule.powerBar.setBackground = UnitFramesModule:getDefaultOrUnitSetter(function(unit, frame)
  frame = frame or UnitFramesModule.frames[unit]
  local db = UnitFramesModule:getDBForUnit(unit)

  frame.Power.bg:SetTexture(db.additionalPowerBar.background)
end)

UnitFramesModule.additionalPowerBar = {}

UnitFramesModule.additionalPowerBar.setTexture = UnitFramesModule:getDefaultOrUnitSetter(function(unit, frame)
  frame = frame or UnitFramesModule.frames[unit]
  local db = UnitFramesModule:getDBForUnit(unit)

  frame.AdditionalPower:SetStatusBarTexture(db.additionalPowerBar.texture)
end)

UnitFramesModule.additionalPowerBar.setBackground = UnitFramesModule:getDefaultOrUnitSetter(function(unit, frame)
  frame = frame or UnitFramesModule.frames[unit]
  local db = UnitFramesModule:getDBForUnit(unit)

  frame.AdditionalPower.bg:SetTexture(db.additionalPowerBar.background)
end)

-- Positioning functions.
UnitFramesModule.positioning = {}

-- oUF Layout functions.
UnitFramesModule.layouts = {
  unit = {},
}

function UnitFramesModule.layouts.default(self, unit)
  -- idTweaks:Print('UnitFramesModule.layouts.default', unit, self)

  -- TODO: Make the Name Text be the Spell Name when Casting
  -- TODO: Threat
  -- TODO: Icons

  local db = UnitFramesModule:getDBForUnit(unit)

  -- Background
  self.background = self:CreateTexture(nil, 'BACKGROUND')
  self.background:SetAllPoints(self)
  self.background:SetColorTexture(0, 0, 0, 0.5)

  -- Health Bar
  local Health = CreateFrame('StatusBar', nil, self)
  Health.colorReaction = true
  Health.colorClass = true
  self.Health = Health

  -- Health Bar Background
  local HealthBG = Health:CreateTexture(nil, 'BORDER')
  HealthBG:SetAllPoints(Health)
  Health.bg = HealthBG

  -- Power Bar
  local Power = CreateFrame('StatusBar', nil, self)
  Power.frequentUpdates = true
  Power.displayAltPower = true
  Power.colorPower = true
  self.Power = Power

  -- Power Bar Background
  local PowerBG = Health:CreateTexture(nil, 'BORDER')
  PowerBG:SetAllPoints(Power)
  Power.bg = PowerBG

  -- Additional Power Bar
  -- local AdditionalPower = CreateFrame('StatusBar', nil, self)
  -- AdditionalPower.colorPower = true
  -- self.AdditionalPower = AdditionalPower

  -- Additional Power Bar Background
  -- local AdditionalPowerBG = Health:CreateTexture(nil, 'BORDER')
  -- AdditionalPowerBG:SetAllPoints(AdditionalPower)
  -- AdditionalPower.bg = AdditionalPowerBG

  -- Buffs
  local Buffs = CreateFrame('Frame', nil, self)
  Buffs.num = 16
  self.Buffs = Buffs

  -- Debuffs
  local Debuffs = CreateFrame('Frame', nil, self)
  Debuffs.num = 16
  self.Debuffs = Debuffs

  -- Tags
  local textPadding = 5
  local NameText = self.Health:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
  NameText:SetFont(NameText:GetFont(), 9)
  NameText:SetTextColor(1, 1, 1, 1)
  NameText:SetPoint(ML, self.Health, ML, textPadding, 0)
  NameText:SetHeight(9)
  self:Tag(NameText, '[name][ >leader][ >status][ >threatcolor][threat]|r')
  self.NameText = NameText

  local HealthText = self.Health:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
  HealthText:SetFont(HealthText:GetFont(), 9)
  HealthText:SetTextColor(1, 1, 1, 1)
  HealthText:SetPoint(MR, self.Health, MR, -textPadding, 0)
  HealthText:SetHeight(9)
  self:Tag(HealthText, '[curhp]')
  self.HealthText = HealthText

  oUF.Tags.Methods['runes'] = function(unit, realUnit)
    if unit ~= 'player' then return end
    local amount = 0
    for i=1,6 do
      local _, _, ready = GetRuneCooldown(i)
      if ready == true then
        amount = amount + 1
      end
    end
    return amount
  end

  -- * Death Knight:  PowerBar and [runes]
  -- * Demon Hunter:  PowerBar
  --   Druid:         PowerBar and [cpoints] (Cat) and AdditionalPowerBar (Mana for Moonkin)
  -- * Mage           PowerBar and [arcanecharges]
  -- * Monk           PowerBar and [chi]
  -- * Paladin        PowerBar and [holypower]
  --   Priest         Mana ???
  -- * Rogue          PowerBar and [cpoints]
  --   Shaman         PowerBar and AdditionalPowerBar (Mana)
  -- * Warlock        PowerBar and [soulshards]
  -- * Warrior        PowerBar

  local ClassPowerText = self.Power:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
  ClassPowerText:SetFont(ClassPowerText:GetFont(), 9)
  ClassPowerText:SetTextColor(1, 1, 1, 1)
  ClassPowerText:SetPoint(ML, self.Power, ML, textPadding, 0)
  ClassPowerText:SetHeight(9)
  self:Tag(ClassPowerText, '[runes][arcanecharges][chi][holypower][soulshards][cpoints]')
  self.ClassPowerText = ClassPowerText

  local PowerText = self.Power:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
  PowerText:SetFont(PowerText:GetFont(), 9)
  PowerText:SetTextColor(1, 1, 1, 1)
  PowerText:SetPoint(MR, self.Power, MR, -textPadding, 0)
  PowerText:SetHeight(9)
  self:Tag(PowerText, '[curpp]')
  self.PowerText = PowerText

  UnitFramesModule.size:positionElements(unit, self)
  UnitFramesModule.size:positionFrame(unit, self)
  UnitFramesModule.healthBar:setTexture(unit, self)
  UnitFramesModule.healthBar:setBackground(unit, self)
  UnitFramesModule.powerBar:setTexture(unit, self)
  UnitFramesModule.powerBar:setBackground(unit, self)
  -- UnitFramesModule.additionalPowerBar:setTexture(unit, self)
  -- UnitFramesModule.additionalPowerBar:setBackground(unit, self)

  if(UnitFramesModule.layouts.unit[unit]) then
    return UnitFramesModule.layouts.unit[unit](self)
  end
end

function UnitFramesModule.layouts.factory(self)
  -- idTweaks:Print('UnitFramesModule.layouts.factory')

  self:SetActiveStyle('idTweaks')

  UnitFramesModule.frames.player = self:Spawn('player')
  UnitFramesModule.frames.pet = self:Spawn('pet')
  UnitFramesModule.frames.target = self:Spawn('target')
  UnitFramesModule.frames.targettarget = self:Spawn('targettarget')

  -- TODO: Enable Party
  -- oUF:SpawnHeader(overrideName, overrideTemplate, visibility, attributes ...)
  -- UnitFramesModule.frames.party = self:SpawnHeader(nil, nil, 'raid,party,solo',
  --   -- http://wowprogramming.com/docs/secure_template/Group_Headers
  --   -- Set header attributes
  --   'showParty', true,
  --   'showPlayer', true,
  --   'yOffset', -0
  -- )
end
